// C Header File
// Created 11/12/2003; 8:21:42 PM

// How far in advance the AI believes a property is worth
#ifdef ADVANCED_AI
const char movesInAdvance[] = {6,6,6,6,6,6,6,6};

/*
How To Win at Monopoly from http://boardgames.about.com/cs/monopoly/ht/win_monopoly.htm
1.	Know the odds. Most players make a complete lap around the board in five turns; you're likely to roll doubles once in
		those five turns (doubles happen about 17 percent of the time).
2.	Buy smart. Always pick up available properties if: (1) no other player owns one of the same group; (2) the purchase
		would give you two or three of the same group; or (3) it blocks someone else from completing a set.
3.	Remember that Illinois Avenue is the square most often landed on (not including Jail). The B&O Railroad also is
		among those most landed on; Go rounds out the top three.
4.	Railroads are better to own than utilities, but utilities should be purchased if the opportunity arises (especially
		if you can get both of them). Short Line is the least useful railroad to own, because it is visited least often.
5.	Get out of jail quickly early in the game, even if you have to pay the $50. Later on, when moving around the board is
		more dangerous, stay in jail as long as you can.
6.	When you build, get to three houses as quickly as possible. The rent raises significantly between two and three
		houses. For example, rent on Illinois Avenue jumps from $300 to $750.
7.	If you're stuck with low-income properties, build to four houses quickly to create a building shortage, hurting other
		players' chances to build. 24 of 32 houses (or 6 of 12 hotels) could be tied up on just six properties.
8.	Avoid mortgaging properties where you own two or more of the properties. If one property in a group is mortgaged,
		you cannot build on any of the properties in that group.
9.	The second set of properties on all four sides of the board is a better investment. One reason: houses and hotels
		cost the same to build as for the first set of properties, but the rent is higher.
10.	The trio of orange properties is an excellent monopoly to own because of their relationship to Jail. A roll of 6 or
		8 (two of the most common rolls) from Jail lands you on an orange.

Tips:
1.	Show no mercy. If a player is down, eliminate him from the game. Luck plays too big a roll in Monopoly to risk a
		comeback.
2.	Don't forget that you can buy houses, make trades, etc., on your turn or between the turns of any other players.
*/

/*
How to Win Every Time in Monopoly from http://www.ehow.com/how_536_win-every-time.html
4.	Go for the Tennessee, St. James Place, and New York (orange properties) and Illinois, Indiana, and Kentucky (red
		properties). They're more likely to be visited because of their locations on the board.
5.	Try to get players to sell you property instead of mortgaging when they owe you money. Heavily mortgaged properties
		can be bought cheaply by other players who land on them.
6.	Build up to three houses on each of your properties, but not more than that, for the best return.
*/


// The following calculations are from http://www.geocities.com/stuartthiel/marvingardens.html

// Expected revenue per rival-roll, assuming ownership of color-group
// Each multiplied by 256 for accuracy: (propReturnValue[][] >> 8)
// The vertical numbers stands for:
//			rent (x2)
//			1 house
//			2 houses
//			3 houses
//			4 houses
//			HOTEL
const unsigned int propReturnValue[6][22] = {
	{  21,  42,  65,  67,  89, 131, 111, 149, 192, 202, 230, 240, 236, 306, 286, 283, 298, 335, 329, 338, 368,  636},
	{   1,  53, 113, 117, 171, 276, 227, 321, 429, 454, 524, 550, 538, 715, 663, 656, 695, 786, 772, 853, 870, 1220},
	{  52, 211, 389, 401, 452, 878, 732,1015,1269,1342,1479,1567,1535,2195,2041,2018,2136,2409,2368,2611,2530, 3712},
	{ 309, 786,1319,1357,1510,2787,2348,2949,3618,3817,4158,4519,4432,5590,5042,4986,5132,5641,5547,5876,5637, 8745},
	{ 618,1465,1977,2033,2291,3880,3270,4449,4939,5209,5545,5636,5527,6879,6128,6060,6169,6878,6763,7031,6639,10601},
	{1126,2188,2840,2918,3168,4742,4010,5425,6356,6698,7027,6850,6719,8265,7310,7229,7302,8050,7917,8282,7737,12553}
};

// This is where it gets a bit confusing
// Under the array rrOwnership, the bits go in the following order:
//   Bit 1    Bit 2    Bit 3    Bit 4
//  READING   PENNA     B&O   SHORT LINE
// The return value is then calculated according to which rr's are owned
// To make things simplier, the first four are if the player owns one rr, the next six if they own 2
// the next four are if they own 3, and the last is if they own 4 rr's
// Once again, each return value is multiplied by 256 for accuracy: (rrReturnValue[] >> 8)
const unsigned char rrOwnership[] =
				{0b0000,0b1000,0b0100,0b0010,0b0001,0b1100,0b1010,0b1001,0b0110,0b0101,0b0011,0b1110,0b1101,0b1011,0b0111,0b1111};
const unsigned int rrReturnValue[] = {0,179,168,185,146,686,717,643,696,622,653,2071,1926,1987,1946,5217};

// Utilities, (much easier than RR's)
// First one is for electric company, second is for water works, and third is for both
// Once again, (utReturnValue[] >> 8)
const unsigned char utOwnership[] = {0,0b10,0b01,0b11};
const unsigned int utReturnValue[] = {0,190,192,943};

#endif






// The AI has access to all of the following variables and functions:
/*
////////////////////
// Game variables //
////////////////////
// Example use: game.stillPlaying[1];
// Please note: game.turn is always one less than the player number, ie for player 1, game.turn will be 0
// For all owners, it is the same as the player number. To see if the current player is the owner of a property, use
// if(game.turn + 1 == owner[SOME_PROP])


typedef struct {
  enum Bool stillPlaying[8]; // If the player is still playing
  enum Bool aiOn[8]; // If the AI is turned on for that player
  int inJail[8]; // If player is in jail
  int position[8]; // Current position
  int money[8]; // Amount of money
  unsigned int getOutFree[8]; // Number of getOutFree cards the players have

  int turn; // Current turn

  unsigned int rolled_1; // Value of first die
  unsigned int rolled_2; // Value of second die
  unsigned int numDoubles; // Number of doubles a player has rolled in a row

  int owner[22]; // Owner of property
  int mortgaged[22]; // If property is mortgaged
  int numHouses[22]; // Number of houses on property
  int rrOwner[4];
  int rrMortgaged[4];
  int utOwner[2];
  int utMortgaged[2];

} SAVED_GAME;



//////////////////////
// Global Variables //
//////////////////////

// Lookup table to convert from 40 squares to the proper position in the property arrays
// -1 is for RR's, and -2 is for Utilities
// Example, convert[3] converts the third square (Baltic Ave) to the second element in property arrays
// houseCost[convert[3]] gives the number of houses on that property
int convert[]= {0,0,0,1,0,-1,2,0,3,4,0,5,-2,6,7,-1,8,0,9,10,0,11,0,12,13,-1,14,15,-2,16,0,17,0,18,19,-1,0,20,0,21};


const int rent[6][22] = {// Rent cost for all of the properties
  {   2,   4,   6,   6,   8,  10,  10,  12,  14,  14,  16,  18,  18,  20,  22,  22,  24,  26,  26,  28,  35,  50},
  {  10,  20,  30,  30,  40,  50,  50,  60,  70,  70,  80,  90,  90, 100, 110, 110, 120, 130, 130, 150, 175, 200},
  {  30,  60,  90,  90, 100, 150, 150, 180, 200, 200, 220, 250, 250, 300, 330, 330, 360, 390, 390, 450, 500, 600},
  {  90, 180, 270, 270, 300, 450, 450, 500, 550, 550, 600, 700, 100, 750, 800, 800, 850, 900, 900,1000,1100,1400},
  { 160, 320, 400, 400, 450, 625, 625, 700, 750, 750, 800, 875, 875, 925, 975, 975,1025,1100,1100,1200,1300,1700},
  { 250, 450, 550, 550, 600, 750, 750, 900, 950, 950,1000,1150,1150,1200,1150,1150,1200,1275,1275,1400,1500,2000}};
int houseCost   [22] = {50,50, 50, 50, 50,100,100,100,100,100,100,150,150,150,150,150,150,200,200,200,200,200};
int mortgageCost[22] = {30,30, 50, 50, 60, 70, 70, 80, 90, 90,100,110,110,120,130,130,140,150,150,160,175,200};
int buyCost     [22] = {60,60,100,100,120,140,140,160,180,180,200,220,220,240,260,260,280,300,300,320,350,400};

int column;							// Current column. 0 & 1 are player and partner. 3 is the middle (arrows)
int options[2][40];			// Which options are open for them to trade. These are displayed on the screen
enum Bool stats[2][40];	// If the corresponding option is selected to be traded


///////////////
// Functions //
///////////////

enum Bool canAddHouse(int prop);                       // Determines if a player can add a house on a property group
enum Bool canRemoveHouse(int prop);                    // Determines if a player can remove a house on a property group
void Exit() __attribute__ ((noreturn));                // Exits the program, turns off grayscale, cleans up
enum Bool groupOwner(int pos);                         // Determines if a player is the owner of a group
int higherHouse(int h1,int h2);                        // Returns the number of houses of the property that has the most
int isOwner(int pos, int player);                      // Checks if player is owner
int lowerHouse(int h1,int h2);                         // Retuns the number of houses of the property that has the least
int payDouble(int pos);                                // If the player owns all property on that group
int rrCalc(int num);                                   // Serves as a lookup (table) for railroads
int rrRent(int curRR);                                 // Determines the amount of rent a person has to pay for a railroad
void trade(void);                                      // Allows a user to trade property and money with other players
inline int utCalc(int num);                            // Serves as a lookup for utilities
*/


int handleMainMenu()
{

  /* RETURNS:
      OPTION_ROLL        roll dice
      OPTION_STATS       stats
      OPTION_TRADE       trade
      OPTION_PROPERTY    property
      OPTION_FORFEIT     forfeit
      OPTION_SAVE        save
  */

  // If the AI is the owner of the property, and owns the entire group, and the houses aren't full,
  // and has the money, have it buy some houses. If not, see if it wants to trade.
  // Otherwise, roll the dice -- it will never need check property cards (or forfeit ;) )
  int i;
  if((isOwner(game.position[game.turn],game.turn+1))&&
		(groupOwner(game.position[game.turn]))&&
    (game.numHouses[convert[game.position[game.turn]]] < 5)&&
    (game.money[game.turn] > houseCost[convert[game.position[game.turn]]])&&
    (game.dontBuyHouses[game.turn] == 0)) {

      // Check to see how many houses the AI can afford
      for(i = 0; i < 15 * houseCost[convert[game.position[game.turn]]]; i += houseCost[convert[game.position[game.turn]]])
        if(i > game.money[game.turn]) break; // Once they can't afford this many houses, stop

      if(game.dontBuyHouses[game.turn])
        game.dontBuyHouses[game.turn] = 0;
      else {
	      game.houseToBuy = game.position[game.turn];
	      game.numHousesToBuy = i - 1;
	    }

      return OPTION_PROPERTY;
  }
  // If the AI shouldn't buy houses, see if it wants to trade (OPTION_ TRADE)
  return OPTION_ROLL;
}


int propertyInfo(unsigned char menuOptions, int curProp, int numHouses)
{
	/* RETURNS:
	    OPTION_LEFT   Move Forewards
	    OPTION_RIGHT  Move Backwards
      OPTION_1      Mortgage/Unmortgage
      OPTION_2      Add a house
      OPTION_3      Remove a house
      OPTION_4      Cancel
  */

  if(!game.dontBuyHouses[game.turn]) {
    if(numHouses) // If the AI can add a house, do it
      return OPTION_2;

    if(curProp < game.houseToBuy) return OPTION_LEFT;

  }

	// In case the AI keeps going in an endless loop to buy some houses, stop it
  game.dontBuyHouses[game.turn] = 1;

  return OPTION_4;
}

int buyRR()
{
	/* RETURNS:
	    OPTION_BUY_IT      Buy it
	    OPTION_LEAVE_IT    Leave it
	*/

  if(game.money[game.turn] - game.saving[game.turn] >= 200) return OPTION_BUY_IT;
	return OPTION_LEAVE_IT;
}

int buyUtility()
{
	/* RETURNS:
	    OPTION_BUY_IT      Buy it
	    OPTION_LEAVE_IT    Leave it
	*/

  if(game.money[game.turn] - game.saving[game.turn] >= 200) return OPTION_BUY_IT;
	return OPTION_LEAVE_IT;
}

int buyProperty()
{
	/* RETURNS:
	    OPTION_BUY_IT      Buy it
	    OPTION_LEAVE_IT    Leave it
	*/

  if(game.money[game.turn] - game.saving[game.turn] >= buyCost[convert[game.position[game.turn]]]) return OPTION_BUY_IT;
	return OPTION_LEAVE_IT;
}

int inDebt()
{
	/* RETURNS:
	    OPTION_1            Property
	    OPTION_2            Trade
	    OPTION_3            Forfeit
	*/
	return OPTION_3;
}

int taxDay()
{
	/* RETURNS:
			OPTION_1						Pay $200
			OPTION_2						10% of value
	*/
	// Add up the amount of the 10%, then return the lowest-cost option
	int i;
	unsigned int total;
	total = game.money[game.turn];
	for(i = 0; i < 22; i++) if(game.owner[i] == game.turn + 1) total += buyCost[i];
	for(i = 0; i < 4; i++) if(game.rrOwner[i] == game.turn + 1) total += 200;
	if(game.utOwner[0] == game.turn + 1) total += 75;
	if(game.utOwner[1] == game.turn + 1) total += 75;
	// When comparing, I just kept the Total multiplied by 10 instead of taking 10%. This
	// prevents me from having to do division
	return (total < 2000 ? OPTION_2: OPTION_1);
}

int inJail()
{
	/* RETURNS:
			OPTION_1						Roll
			OPTION_2						Pay $50
			OPTION_3						Use Card
	*/
	// If the AI is able to, let them use a card
	// If not, if they have more than $500, then let them pay $50
	// Otherwise, make them roll the dice
	if(game.getOutFree[game.turn]) return OPTION_3;
	if(game.money[game.turn] > 500) return OPTION_2;
	return OPTION_1;
}

int tradepartner(int available)
{
	/* RETURNS:
			OPTION_1						Player 1
			OPTION_2						Player 2
			OPTION_3						Player 3
			OPTION_4						Player 4
			OPTION_5						Player 5
			OPTION_6						Player 6
			OPTION_7						Player 7
			OPTION_8						Player 8
	*/
	// The variable "available" tells what players are open
	// to trade with. If only the first player were open, the
	// value would be 0b10000000. If the third and seventh players
	// were available, then the value would be 0b00100010.

	int i;

	// Trade with the first player available
	for(i = 0; i < 7; i++)
		if((available << i) > 0b10000000) return i + 1;

	return OPTION_1;
}

int tradeCenter()
{
	/* RETURNS:
			OPTION_LEFT					Move Cursor Left
			OPTION_RIGHT				Move Cursor Right
			OPTION_1						Finish trading transaction
	*/

	return OPTION_1;
}

int selectTradeOption(int availableOptions, int column)
{
	/* RETURNS:
			OPTION_1						Option 1/Scroll Up
			OPTION_2						Option 2
			OPTION_3						Option 3
			OPTION_4						Option 4
			OPTION_5						Option 5
			OPTION_6						Option 6
			OPTION_7						Option 7
			OPTION_8						Option 8/Scroll Down
			OPTION_LEFT					Move Cursor Left
			OPTION_RIGHT				Move Cursor Right
	*/
	// Make sure to get out of there as quickly as possible
	if(!column) return OPTION_RIGHT;
	else if(column == 1) return OPTION_LEFT;
	else return OPTION_1;
}
/*
int column;							// Current column. 0 & 1 are player and partner. 3 is the middle (arrows)
int options[2][40];			// Which options are open for them to trade. These are displayed on the screen
enum Bool stats[2][40];	// If the corresponding option is selected to be traded


									if(options[column][i] == 3) {
										game.money[lastOptions[0]] -= moneyToTrade;
										game.money[lastOptions[1]] += moneyToTrade;
									} else if(options[column][i] == 4) {
										game.getOutFree[lastOptions[0]]--;
										game.getOutFree[lastOptions[1]]++;
									} else if(options[column][i] <= 8) {
										game.rrOwner[options[column][i] - 5] = lastOptions[1] + 1;
									} else if(options[column][i] <= 10) {
										game.utOwner[options[column][i] - 9] = lastOptions[1] + 1;
									} else if(options[column][i] > 10) {
										game.owner[options[column][i] - 11] = lastOptions[1] + 1;
									}*/

int agreeToTradeIt(int turn, unsigned long int moneyToTrade)
{
	/* RETURNS:
			FALSE								No
			TRUE								Yes
	*/
#ifdef ADVANCED_AI
	int col = 0;
	int row = 0;
	unsigned long int worth[2] = {0,0}; // Keep the numbers shifted up by 8 for more exact numbers
	int rr = 0, ut = 0;

	for(; col < 2; col++) { // Cycle through both sides
		for(row = 0; row < 40 && options[col][row]; row++) // Go through each option
		{
			if(stats[col][row]) { // If the option is checked, figure out the value
				if(options[col][row] == 3) // Money
					worth[col] += (moneyToTrade << 8);
				else if(options[col][row] == 4) // Get Out Free
					worth[col] += (30 << 8);
				else if(options[col][row] <= 8) // RR's
					rr += (1 << (options[col][row] - 5)); // Add the RR to the list of rr's
				else if(options[col][row] <=10) // UT's
					ut += (1 << (options[col][row] - 9)); // Add the UT to the list of ut's



			}
		}
		// Now check to see the value of the rr's, ut's, and props

		// Match the same rr ownership
		// Then add the value of that combination of rr's
		for(row = 0; row < 16; row++)
			if(rr == rrOwnership[row])
				worth[col] += (unsigned long)(rrReturnValue[row] << movesInAdvance[turn]);

		// Do the same thing to the Utilities
		for(row = 0; row < 3; row++)
			if(ut == utOwnership[row])
				worth[col] += utReturnValue[row] * movesInAdvance[turn];
	}

	//
	memset(GetPlane(LIGHT_PLANE), 0, LCD_SIZE);
	memset(GetPlane(DARK_PLANE), 0, LCD_SIZE);
	char buffer[30];
	sprintf(buffer, "Value 1: %ld", worth[0]);
	DrawBlackStr(10,10,buffer);
	sprintf(buffer, "Value 2: %ld", worth[1]);
	DrawBlackStr(10,30,buffer);

	pause(); pause();
#endif
	return FALSE;
}